@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014  Innocent De Marchi
+ * Copyright (C) 2000-2016  Innocent De Marchi
  * email: tangram.peces@gmail.com
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,8 +18,10 @@
  */
 #include <QtGui>
 #include <QIcon>
-
-
+#include <QtWidgets/QApplication>
+#include <QtWidgets/QInputDialog>
+#include <QtWidgets/QFileDialog>
+#include <QtWidgets/QMessageBox>
 #include "creaconcurs.h"
 
 
