@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014  Innocent De Marchi
+ * Copyright (C) 2000-2016  Innocent De Marchi
  * email: tangram.peces@gmail.com
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 #ifndef DESAFIGURA_H
 #define DESAFIGURA_H
 
-#include <QDialog>
+#include <QtWidgets/QDialog>
 #include "ui_frmDesaFigura.h"
 #include "configuracio.h"
 
