@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014  Innocent De Marchi
+ * Copyright (C) 2000-2016  Innocent De Marchi
  * email: tangram.peces@gmail.com
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,6 +17,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 #include <QtGui>
+#include <QtWidgets/QGraphicsScene>
+#include <QtWidgets/QGraphicsSceneMouseEvent>
+#include <QtWidgets/QStyleOptionGraphicsItem>
+#include <QtWidgets/QStyle>
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -129,17 +133,18 @@ QRectF Peca::boundingRect() const
 void Peca::mousePressEvent(QGraphicsSceneMouseEvent *event){
  if(!pecaMostra){
   if (shape().contains(event->pos()) && !pecaMostra){
-
       /* Activam les funcions tàctils si es donen les condicions
        */
       if(distanciaEntrePunts(event->pos(),centreRectanglePeca())>6.0  &&
             isSelected() && event->button()==Qt::LeftButton ){
           movimentPecaTactil=true;
           setSelected(true);
+          controlPecaSeleccionada();
       }
       else {
           movimentPecaTactil=false;
-          setSelected(false);}
+          setSelected(false);
+      }
 
 	//Controlam el botó que s'ha clicat
 	//Amb el botó esquerra+ Ctrl---> simetria de la peça
@@ -150,7 +155,6 @@ void Peca::mousePressEvent(QGraphicsSceneMouseEvent *event){
                 /* 15/10/11
                 afegit el control de superposició */
                 if(!pecaEnLimits() || pecaSuperposada()) {
-               // if(!pecaEnLimits()){
                     event->ignore();
                     simetriaPeca();
                 }
@@ -187,12 +191,12 @@ void Peca::mousePressEvent(QGraphicsSceneMouseEvent *event){
 	//Si es clica el botó mitjancer, es fa la simetria de la peça
 	else if (event->button()==Qt::MidButton){
             setSelected(true);
+            controlPecaSeleccionada();
             simetriaPeca();
             //comprova si la peça està en els limits de la zona de joc
             /* 15/10/11
             afegit el control de superposició */
             if(!pecaEnLimits() || pecaSuperposada()) {
-               // if(!pecaEnLimits()){
                     event->ignore();
                     simetriaPeca();
                 }
@@ -215,6 +219,7 @@ void Peca::mousePressEvent(QGraphicsSceneMouseEvent *event){
                }
             else { //rotació de la peça amb angle fix (amb el botó dret)
                 //QPointF finalGir=event->pos()-centrePeca();
+                controlPecaSeleccionada();
                 QPointF finalGir=event->pos()-centreRectanglePeca();
                 float angleFinal=atan2(finalGir.y(),finalGir.x());
                 int acelera=1;
@@ -279,6 +284,7 @@ void Peca::mousePressEvent(QGraphicsSceneMouseEvent *event){
 
 void Peca::mouseMoveEvent ( QGraphicsSceneMouseEvent *event){
 if(!pecaMostra){
+    controlPecaSeleccionada();
   /* 23/07/11
      Això feia que en moure la peça ràpidament,
      sovint queda aturada. Pareix què també ara permet
@@ -300,8 +306,9 @@ if(!pecaMostra){
         /*Com què la peça està superposada o fora del límits
          *es desfà el moviment i es permet que la peça giri
          */
-           event->ignore();
+           //
            setPos(darreraPos);
+           event->ignore();
            /* 03/03/14 innecessari*/
           /* if (angleGirPeca==ROTACIO_EXPERT){
                movimentPeca=2;
@@ -317,28 +324,34 @@ if(!pecaMostra){
         float angleInici=atan2(iniciGir.y(),iniciGir.x());
         float angleFinal=atan2(finalGir.y(),finalGir.x());
 
-        if (angleInici<0){angleInici=angleInici+2*PI;}
-        if (angleFinal<0){angleFinal=angleFinal+2*PI;}
+       // if (angleInici<0){angleInici=angleInici+2*PI;}
+       // if (angleFinal<0){angleFinal=angleFinal+2*PI;}
         //Si la tecla crtl està pitjada, llavors
         //la rotació es fa més "lenta"
         double precisio=1.0;
         if (event->modifiers().testFlag ( Qt::ControlModifier )){
                   precisio=0.2;}
-        rotaPeca(precisio*(angleFinal-angleInici));
-        /* 15/10/11
-        afegit el control de superposició */
-        if(!pecaEnLimits() || pecaSuperposada() ){
+        //28/12/15 Eliminat el següent if atès que la comprovació
+        //de la superposició ja ho fa el proc. rotapeca()
+        if(!rotaPeca(precisio*(angleFinal-angleInici),true)){
+            event->ignore();
+            movimentPeca=1;
+          }
+            ;
+        // 15/10/11
+        //afegit el control de superposició
+       /* if(!pecaEnLimits() || pecaSuperposada() ){
            event->ignore();
-           rotaPeca(precisio*(-angleFinal+angleInici));
+           rotaPeca(precisio*(-angleFinal+angleInici),true);
            movimentPeca=1;
-         };
+         };*/
        } // final else if moviment expert
 
       /* 15/10/11
          Afegit això per permetre que les peces girin automàticament
          quan xoquin
         */
-      else if (angleGirPeca != ROTACIO_EXPERT && !movimentPecaTactil){
+    else if ( angleGirPeca != ROTACIO_EXPERT && !movimentPecaTactil ){
           //rotació de la peça amb angle fix
          // QPointF finalGir=event->pos()-centrePeca();
           QPointF finalGir=event->lastPos()-centrePeca();
@@ -355,29 +368,31 @@ if(!pecaMostra){
           //clic
           double girFet;
           if (angleFinal>PI){
-              rotaPeca(acelera*angleGirPeca*PI/180);
               girFet=acelera*angleGirPeca*PI/180;
+              rotaPeca(girFet,true);
           }
           else {
-              rotaPeca(-acelera*angleGirPeca*PI/180);
               girFet=-acelera*angleGirPeca*PI/180;
+              rotaPeca(girFet,true);
           }
           //comprova si la peça està en els limits de la zona de joc
           /* 15/10/11
-             afegit el control de superposició */
-          if(!pecaEnLimits() || pecaSuperposada()) {
+             afegit el control de superposició
+          28/12/15 Eliminat ja que la comprovació de la superposició
+          ja es fa a rotapeca()*/
+          /*if(!pecaEnLimits() || pecaSuperposada()) {
               //if(!pecaEnLimits()){
               event->ignore();
               rotaPeca(-girFet);
              // movimentPeca=1;
-          }
+          }*/
           movimentPeca=1;
           movimentPecaTactil=false;
       }
  /*
    }//final if contains  23/07/11
   else {
-     //Això per evitar que la peça quedi "seleccionada" quan el curso està fora de la peça
+     //Això per evitar que la peça quedi "seleccionada" quan el cursor està fora de la peça
      setSelected(false);
      event->ignore();
     // setZValue(-1);
@@ -399,7 +414,18 @@ void Peca::mouseReleaseEvent(QGraphicsSceneMouseEvent *event){
    if ( !event->modifiers().testFlag ( Qt::ControlModifier )){
        QPointF distancia;
        distancia= aferraPeca();
+       QPointF posicioActual=pos();
        setPos(pos()-distancia);
+       /*04/08/15 Afagit per evitar que les peces
+        * quedin una mica superposades i millorar el tema de les dades
+        * de posicions de les peces per a la generació automàtica de figures*/
+       float reduccio=1.0;
+       int contador=0;
+       while(pecaSuperposada(false) && contador<10){
+           setPos(posicioActual-distancia*reduccio);
+           reduccio=reduccio-0.01;
+           contador++;
+       }
        /* 15/10/11
           Es controla que la peça no quedi superposada
           allunyant-la desprès d'aferrar-la.
@@ -425,11 +451,15 @@ void Peca::mouseReleaseEvent(QGraphicsSceneMouseEvent *event){
  QGraphicsItem::mouseReleaseEvent(event);
 }//final Peca::mouseReleaseEvent
 
-/* Controla si la peça està en els límits del joc o/i superposada
- * a una altra peça i, si cal, desfà el moviment
+/* Assegura que la peça actualment seleccionada amb el ratolí
+ * sigui l'única seleccionada
  */
-void Peca::controlaMovimentPecaLimitsSuperposicio(){
-
+void Peca::controlPecaSeleccionada(){
+ for(int i=0;i<=conjuntDePeces->arrayPeces.size()-1;i++){
+  if(!(this->indexEnElConjunt == conjuntDePeces->arrayPeces[i]->indexEnElConjunt) ){
+    conjuntDePeces->arrayPeces[i]->setSelected(false);
+  }
+ }
 }
 
 /*
@@ -441,12 +471,14 @@ void Peca::controlaMovimentPecaLimitsSuperposicio(){
   (amb el procediment aferraPeca2)
 */
 QPointF Peca::aferraPeca(){
+
  QVector<QPointF> aquestaPeca;
  QVector<QPointF> altraPeca;
  QPointF desplacament, despActual;
  float distancia, distanciaActual;
  puntsVertexPeca(aquestaPeca);
  desplacament=QPointF(0,0);
+ //QPointF posicioInicial=pos();
  bool aferrada=false;
  for(int i=0;i<=conjuntDePeces->arrayPeces.size()-1;i++){
      if(!(this->indexEnElConjunt == conjuntDePeces->arrayPeces[i]->indexEnElConjunt) ){
@@ -457,9 +489,9 @@ QPointF Peca::aferraPeca(){
                 despActual=aquestaPeca[j]-altraPeca[k];
                 distanciaActual=sqrt(pow(despActual.x(),2)+pow(despActual.y(),2));
            /* 20/03/14 Afegit && !pecaSuperposada(false): alguns casos podian quedar peces
-            * superposades
-            */
+            * superposades */
                 if( (distanciaActual<distancia) && (distanciaActual<=5.0) && !pecaSuperposada(false)){
+                   //!pecaSuperposadaEscala(false,0.90)){
                    distancia=distanciaActual;
                    desplacament=despActual;
                    aferrada=true;
@@ -468,7 +500,7 @@ QPointF Peca::aferraPeca(){
         }
      }
     }
- if(aferrada){
+ if(aferrada){     
      return desplacament;
    }
  else return aferraPeca2();
@@ -496,6 +528,7 @@ QPointF Peca::aferraPeca2(){
                            (centreRectanglePeca()+pos());
      if(indexEnElConjunt != conjuntDePeces->arrayPeces[i]->indexEnElConjunt){
          while( (distanciaRecorreguda<8.0) &&  !pecaSuperposada(false) ){
+                //!pecaSuperposadaEscala(false,1.1)){
          setPos(pos()+vectorCentres*vector);
          distanciaRecorreguda=sqrt(pow((posicioInicial-pos()).x(),2)+pow((posicioInicial-pos()).y(),2));
          valor++;
@@ -555,6 +588,30 @@ bool Peca::pecaSuperposada(bool controlInfantil){
   return xoca;
 }//final Peca::pecaSuperposada()
 
+/* El mateix que l'anterior, però es pot fer servir
+ * el paràmetre escala per augmentar o minvar el tamany
+ * de la peça actual*/
+bool Peca::pecaSuperposadaEscala(bool controlInfantil, float escala){
+    //això restringeix la funció als nivells
+    //iniciat i expert
+    if( (nivellJoc==NIVELL_INFANTIL) &&
+        controlInfantil){return false;}
+    bool xoca=false;
+    for(int j=0;j<conjuntDePeces->arrayPeces.size();j++){
+        if (conjuntDePeces->arrayPeces[j] != this){
+            if(conjuntDePeces->arrayPeces[j]->shape().translated(
+                  conjuntDePeces->arrayPeces[j]->pos())
+                .intersects(
+                    this->shapeEscala(escala).translated(
+                          this->pos()))){
+            xoca=true;
+           // qDebug("La peça xoca amb escala!");
+            }
+        }
+    }
+  return xoca;
+}//final Peca::pecaSuperposada()
+
 /* Controla si la peça està superposada a
  * les peçes passades per paràmetre
  * Si hi ha una peça que no és de les passades per paràmetre
@@ -562,22 +619,29 @@ bool Peca::pecaSuperposada(bool controlInfantil){
  */
 bool Peca::pecaSuperposada(QList<int> indexPeces){
 bool xoca=false;
-
+/*07/09/15 Afegit la segona condició al 1r if
+ * i comentat el 3er if*/
 for(int j=0;j<conjuntDePeces->arrayPeces.size();j++){
- if (conjuntDePeces->arrayPeces[j] != this){
+ if (  (conjuntDePeces->arrayPeces[j] != this)  ){ //&&
+       //(indexPeces.contains(j)) ){
  if(conjuntDePeces->arrayPeces[j]->shape().translated(
        conjuntDePeces->arrayPeces[j]->pos())
      .intersects(
          this->shape().translated(
                this->pos()))){
- if(indexPeces.contains(j) || !pecaEnLimits()){
- xoca=true;}
- else conjuntDePeces->arrayPeces[j]->colocaPecaAtzar();
- xoca=true;
- }
+     if(indexPeces.contains(j) || !pecaEnLimits()){
+            xoca=true;}
+     else {
+     /* Si està superposada a una peça que no és de les col·locades,
+      * es mou la peça i es retorna false per permetre que segueixi
+      * el gir en el mateix sentit
+      */
+         conjuntDePeces->arrayPeces[j]->colocaPecaAtzar();
+         xoca=false;}
+    // xoca=true;
+    }
  }
 }
-
 /*for(int j=0;j<indexPeces.size();j++){
  if(conjuntDePeces->arrayPeces[indexPeces.value(j)]->shape().translated(
        conjuntDePeces->arrayPeces[indexPeces.value(j)]->pos())
@@ -590,6 +654,28 @@ for(int j=0;j<conjuntDePeces->arrayPeces.size();j++){
 return xoca;
 }
 
+/* Igual que l'anterior, però amb el paràmetre escala
+ */
+bool Peca::pecaSuperposadaEscala(QList<int> indexPeces, double escala){
+bool xoca=false;
+
+for(int j=0;j<conjuntDePeces->arrayPeces.size();j++){
+ if (conjuntDePeces->arrayPeces[j] != this){
+ if(conjuntDePeces->arrayPeces[j]->shape().translated(
+       conjuntDePeces->arrayPeces[j]->pos())
+     .intersects(
+         this->shapeEscala(escala).translated(
+               this->pos()))){
+     if(indexPeces.contains(j) || !pecaEnLimits()){
+         xoca=true;
+       }
+    }
+ }
+}
+return xoca;
+}
+
+
 
 /*
 Retorna una cadena amb les dades de la peça
@@ -652,7 +738,7 @@ QColor Peca::getColorLimits() const
   */
 void Peca::setAngleRotacio(double angle){
    //reiniciaAnglesVertex();
-	angleRotacio=-angle;
+    angleRotacio=-angle;
 	//cal actualitzar els angles dels vertex
 	actualitzaAnglesVertex(angle);
 	//Actualitzam la peça
@@ -662,6 +748,7 @@ void Peca::setAngleRotacio(double angle){
 bool Peca::rotaPeca(double angle, bool controlaLimitsSuperposicio){
    angleRotacio=angleRotacio-angle; 
    actualitzaAnglesVertex(angle);
+   //qDebug("angle %f",angle);
    bool retorn=false;
    /* 02/03/14
     * Això podria estar aqui, però caldria
@@ -1076,6 +1163,7 @@ if (pecaVisible){
       pen.setColor(Qt::red);}
     else  {
       painter->setBrush(colorPeca);
+      //painter->setBrush(Qt::white);
       pen.setColor(colorLimits);
       }
     painter->setPen(pen);    
@@ -1088,16 +1176,18 @@ if (pecaVisible){
        painter->setOpacity(transparencia);
      }
 
-    /* transparent si està superposada
+    /* color negre si està superposada
     if (pecaSuperposada(false)) {
-       painter->setOpacity(transparencia);
+       //painter->setOpacity(transparencia);
        painter->setBrush(Qt::white);
      }*/
 
+    /*Fons blanc si s'ha fet la simetria a la peça
+    if (simetriaBool){
+        painter->setBrush(Qt::white);
+    }*/
+
     painter->drawPath(shape());
-    /*
-    painter->setPen(Qt::black);
-    painter->drawText(centrePeca(),QString("%1").arg(tipusPeca));*/
 
     /* 02/03/14
      * Dibuixa un cercle al centre del rectangle
@@ -1111,6 +1201,10 @@ if (pecaVisible){
     painter->drawEllipse(boundingRect().center(),6,6);
     }
 
+    /* Dibuixa el índex a l'array de peces i el codi de la peça
+painter->setPen(Qt::black);
+painter->drawText(1,1,QString("%1 (%2)").arg(indexEnElConjunt).arg(tipusPeca));*/
+
     /* Dibuixa el rectangle que envolta la peça
      */
    // painter->drawPolyline(boundingRect());
@@ -1336,6 +1430,8 @@ QPointF Peca::generaFiguraAtzar_aferraPeca(QList<int> indexPecesColocades,double
    }
  else if(continua) return generaFiguraAtzar_aferraPeca2(indexPecesColocades);
  return desplacament;
+
+ //return generaFiguraAtzar_aferraPeca2(indexPecesColocades);
 }
 
 QPointF Peca::generaFiguraAtzar_aferraPeca2(QList<int> indexPecesColocades){
@@ -1393,20 +1489,21 @@ bool Peca::generaFiguraAtzar_aferraPecaLentament(QList<int> indexPecesColocades,
          QPointF(0,0)){
     int contador=0;
     int sentitRotacio=pow(-1,qrand()% 2);
-   while( contador<200){
+   //while( contador<200){
+   while( !pecaSuperposadaEscala(indexPecesColocades,1.01) && contador<200){
         /* !pecaSuperposada(indexPecesColocades) ){// && contador<100){*/
         generaFiguraAtzar_mouPeca(pos()-0.01*generaFiguraAtzar_aferraPeca(
                                      indexPecesColocades,distanciaAferraPeca2,continua),
                                      indexPecesColocades,sentitRotacio);
 
         //pausa(2);
-        if(pecaSuperposada(indexPecesColocades)){
+        if(pecaSuperposadaEscala(indexPecesColocades,1.01)){
+       // if(pecaSuperposada(indexPecesColocades)){
             contador=900;
             //qDebug("superposada");
         }
         contador++;
-       // pausa(2);
-       // qDebug("contador++ %d",contador);
+        //qDebug("aferra %d",contador);
     }
    fet=true;
    }
@@ -1418,13 +1515,15 @@ return fet;
 bool Peca::generaFiguraAtzar_separaPecaLentament(QList<int> indexPecesColocades,
                                                  QPointF vectorMoviment){
     movimentPeca=1;
-    int contador=0;
+   // int contador=0;
     QPointF darreraPosicio;
-    while(pecaSuperposada(indexPecesColocades) && contador<100){
+   // while(pecaSuperposada(indexPecesColocades) && contador<100){
+    while (pecaSuperposadaEscala(indexPecesColocades,0.99)){
        setPos(pos()-0.01*vectorMoviment);
        if(darreraPosicio !=pos()){
        darreraPosicio=pos();}
-       contador++;
+     //  contador++;
+       //qDebug("separa %d",contador);
     }
     setPos(darreraPosicio);
     //pausa(5);
@@ -1435,55 +1534,59 @@ return true;
 /* Mou la peça al punt passat per paràmetre seguint
  * el procediment del moviment amb el ratolí
  */
-void Peca::generaFiguraAtzar_mouPeca(QPointF punt, QList<int> indexPecesColocades,int sentitRotacio){
-
-   // qDebug("angleGir %f",angleGirPeca);
+bool Peca::generaFiguraAtzar_mouPeca(QPointF punt, QList<int> indexPecesColocades, int sentitRotacio){
+   bool valorRetorn=true;
     /* La peça es pot moure però no està activat
      * el gir tàctil
+     * 10/09/15 El paràmetre movimentPecaTactil ja no es considera
+     * (per a les peces de mostra no te cap sentit fer-ho servir)
      */
-    if ( (movimentPeca==1) && !movimentPecaTactil ){
+    if ( (movimentPeca==1) ){  //&& !movimentPecaTactil ){
         QPointF darreraPos=pos();
         setPos(punt);
-       // qDebug("en moviment");
+
         /* 15/10/11
-        afegit el control de superposició */
-        if(!pecaEnLimits() || pecaSuperposada(indexPecesColocades)){
-        /*Com què la peça està superposada o fora del límits
+        afegit el control de superposició
+        */
+        if(!pecaEnLimits() || pecaSuperposada(indexPecesColocades) ){
+        /*Com què la peça està superposada o fora dels límits
          *es desfà el moviment i es permet que la peça giri
          */
            setPos(darreraPos);
            movimentPeca=2;
          }
       }
-    else if (  ((movimentPeca==2) || (movimentPecaTactil)) &&  (angleGirPeca==ROTACIO_EXPERT) ) {
-       // int sentitRotacio=pow(-1,qrand()% 2);
-        rotaPeca(angleGirPeca*PI/180*sentitRotacio);
-        //qDebug("en rotacio");
+    /*  Aquesta opció comentada no te cap sentit
+     */
+    //else if (  ((movimentPeca==2) || (movimentPecaTactil)) &&  (angleGirPeca==ROTACIO_EXPERT) ) {
+   // else if (  (movimentPeca==2)  &&  (angleGirPeca==ROTACIO_EXPERT) ) {
+        // rotaPeca(angleGirPeca*PI/180*sentitRotacio);
         /* 15/10/11
         afegit el control de superposició */
-        if(!pecaEnLimits() || pecaSuperposada(indexPecesColocades) ){
-           rotaPeca(-angleGirPeca*PI/180*sentitRotacio);
-         };
-        movimentPeca=1;
-       } // final else if moviment expert
+      //  if(!pecaEnLimits() || pecaSuperposada(indexPecesColocades) ){
+       //    rotaPeca(-angleGirPeca*PI/180*sentitRotacio);
+       //    valorRetorn=false;
+       //  };
+       // movimentPeca=1;
+      // } // final else if moviment expert
 
       /* 15/10/11
          Afegit això per permetre que les peces girin automàticament
          quan xoquin
         */
-      else if ( angleGirPeca != ROTACIO_EXPERT && !movimentPecaTactil){
-         // int sentitRotacio2=pow(-1,qrand()% 2);
-          rotaPeca(angleGirPeca*PI/180*sentitRotacio);
-          //qDebug("en rotacio-2");
+   else if (  (movimentPeca==2) ){ // && (angleGirPeca != ROTACIO_EXPERT) ){  //&& !movimentPecaTactil){
+            rotaPeca(angleGirPeca*PI/180*sentitRotacio);
           //comprova si la peça està en els limits de la zona de joc
           /* 15/10/11
              afegit el control de superposició */
-          if(!pecaEnLimits() || pecaSuperposada(indexPecesColocades)) {
+          if(!pecaEnLimits()  || pecaSuperposada(indexPecesColocades)) {
               rotaPeca(-angleGirPeca*PI/180*sentitRotacio);
+              valorRetorn=false;
           }
-          movimentPeca=1;
-          movimentPecaTactil=false;
+         // movimentPeca=1;
+         // movimentPecaTactil=false;
       }
+ return valorRetorn;
 }
 
 /*
