@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2014  Innocent De Marchi
+ * Copyright (C) 2000-2016  Innocent De Marchi
  * email: tangram.peces@gmail.com
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 #define RELLOTGE_H
 
 #include <QtGui>
-#include <QLabel>
+#include <QtWidgets/QLabel>
 
 
 class QTimer;
